@@ -103,6 +103,18 @@ static REDIRECTS: Lazy<HashMap<&'static str, &'static str>> = Lazy::new(|| {
         ("TRY200", "B904"),
         ("PGH001", "S307"),
         ("PGH002", "G010"),
+        // flake8-trio and flake8-async merged with name flake8-async
+        ("TRIO", "ASYNC1"),
+        ("TRIO1", "ASYNC1"),
+        ("TRIO10", "ASYNC10"),
+        ("TRIO100", "ASYNC100"),
+        ("TRIO105", "ASYNC105"),
+        ("TRIO109", "ASYNC109"),
+        ("TRIO11", "ASYNC11"),
+        ("TRIO110", "ASYNC110"),
+        ("TRIO115", "ASYNC115"),
+        // Removed in v0.5
+        ("PLR1701", "SIM101"),
         // Test redirect by exact code
         #[cfg(any(feature = "test-rules", test))]
         ("RUF940", "RUF950"),
