@@ -9,10 +9,10 @@ use ruff_linter::{
 };
 use ruff_notebook::SourceValue;
 use ruff_source_file::LineIndex;
-use ruff_workspace::resolver::match_any_exclusion;
 
 use crate::{
     edit::{Replacement, ToRangeExt},
+    resolve::is_document_excluded,
     session::DocumentQuery,
     PositionEncoding,
 };
@@ -33,18 +33,12 @@ pub(crate) fn fix_all(
 
     // If the document is excluded, return an empty list of fixes.
     let package = if let Some(document_path) = document_path.as_ref() {
-        if let Some(exclusion) = match_any_exclusion(
+        if is_document_excluded(
             document_path,
-            &file_resolver_settings.exclude,
-            &file_resolver_settings.extend_exclude,
-            Some(&linter_settings.exclude),
+            file_resolver_settings,
+            Some(linter_settings),
             None,
         ) {
-            tracing::debug!(
-                "Ignored path via `{}`: {}",
-                exclusion,
-                document_path.display()
-            );
             return Ok(Fixes::default());
         }
 
@@ -69,8 +63,7 @@ pub(crate) fn fix_all(
     let FixerResult {
         transformed,
         result: LinterResult {
-            has_syntax_error: has_error,
-            ..
+            has_syntax_error, ..
         },
         ..
     } = ruff_linter::linter::lint_fix(
@@ -83,9 +76,9 @@ pub(crate) fn fix_all(
         source_type,
     )?;
 
-    if has_error {
-        // abort early if a parsing error occurred
-        return Err(anyhow::anyhow!("A parsing error occurred during `fix_all`"));
+    if has_syntax_error {
+        // If there's a syntax error, then there won't be any fixes to apply.
+        return Ok(Fixes::default());
     }
 
     // fast path: if `transformed` is still borrowed, no changes were made and we can return early
