@@ -21,6 +21,7 @@ pub(crate) struct ResolvedClientSettings {
     lint_enable: bool,
     disable_rule_comment_enable: bool,
     fix_violation_enable: bool,
+    show_syntax_errors: bool,
     editor_settings: ResolvedEditorSettings,
 }
 
@@ -60,7 +61,7 @@ pub(crate) enum ConfigurationPreference {
 #[derive(Debug, Deserialize, Default)]
 #[cfg_attr(test, derive(PartialEq, Eq))]
 #[serde(rename_all = "camelCase")]
-pub(crate) struct ClientSettings {
+pub struct ClientSettings {
     configuration: Option<String>,
     fix_all: Option<bool>,
     organize_imports: Option<bool>,
@@ -70,6 +71,13 @@ pub(crate) struct ClientSettings {
     exclude: Option<Vec<String>>,
     line_length: Option<LineLength>,
     configuration_preference: Option<ConfigurationPreference>,
+
+    /// If `true` or [`None`], show syntax errors as diagnostics.
+    ///
+    /// This is useful when using Ruff with other language servers, allowing the user to refer
+    /// to syntax errors from only one source.
+    show_syntax_errors: Option<bool>,
+
     // These settings are only needed for tracing, and are only read from the global configuration.
     // These will not be in the resolved settings.
     #[serde(flatten)]
@@ -83,6 +91,7 @@ pub(crate) struct ClientSettings {
 #[serde(rename_all = "camelCase")]
 pub(crate) struct TracingSettings {
     pub(crate) log_level: Option<crate::trace::LogLevel>,
+    /// Path to the log file - tildes and environment variables are supported.
     pub(crate) log_file: Option<PathBuf>,
 }
 
@@ -243,6 +252,11 @@ impl ResolvedClientSettings {
                 },
                 true,
             ),
+            show_syntax_errors: Self::resolve_or(
+                all_settings,
+                |settings| settings.show_syntax_errors,
+                true,
+            ),
             editor_settings: ResolvedEditorSettings {
                 configuration: Self::resolve_optional(all_settings, |settings| {
                     settings
@@ -344,6 +358,10 @@ impl ResolvedClientSettings {
         self.fix_violation_enable
     }
 
+    pub(crate) fn show_syntax_errors(&self) -> bool {
+        self.show_syntax_errors
+    }
+
     pub(crate) fn editor_settings(&self) -> &ResolvedEditorSettings {
         &self.editor_settings
     }
@@ -438,6 +456,7 @@ mod tests {
                 exclude: None,
                 line_length: None,
                 configuration_preference: None,
+                show_syntax_errors: None,
                 tracing: TracingSettings {
                     log_level: None,
                     log_file: None,
@@ -490,6 +509,7 @@ mod tests {
                         exclude: None,
                         line_length: None,
                         configuration_preference: None,
+                        show_syntax_errors: None,
                         tracing: TracingSettings {
                             log_level: None,
                             log_file: None,
@@ -555,6 +575,7 @@ mod tests {
                         exclude: None,
                         line_length: None,
                         configuration_preference: None,
+                        show_syntax_errors: None,
                         tracing: TracingSettings {
                             log_level: None,
                             log_file: None,
@@ -601,6 +622,7 @@ mod tests {
                 lint_enable: true,
                 disable_rule_comment_enable: false,
                 fix_violation_enable: false,
+                show_syntax_errors: true,
                 editor_settings: ResolvedEditorSettings {
                     configuration: None,
                     lint_preview: Some(true),
@@ -632,6 +654,7 @@ mod tests {
                 lint_enable: true,
                 disable_rule_comment_enable: true,
                 fix_violation_enable: false,
+                show_syntax_errors: true,
                 editor_settings: ResolvedEditorSettings {
                     configuration: None,
                     lint_preview: Some(false),
@@ -699,6 +722,7 @@ mod tests {
                     ),
                 ),
                 configuration_preference: None,
+                show_syntax_errors: None,
                 tracing: TracingSettings {
                     log_level: Some(
                         Warn,
@@ -725,6 +749,7 @@ mod tests {
                 lint_enable: true,
                 disable_rule_comment_enable: false,
                 fix_violation_enable: true,
+                show_syntax_errors: true,
                 editor_settings: ResolvedEditorSettings {
                     configuration: None,
                     lint_preview: None,
